package org.rentierdb.objectmanagement.graph.edge;

import java.util.Collection;
import java.util.Map;

/**
 * This interface specifies several CRUD operations executable on IEdges
 * @author Jan-Christoph Klie
 *
 */
public abstract interface IEdgeCRUD {	
	
	//####################Read######################	
	
	/**
	 * Searches the specified edge for an attribute with the specified name and returns if found its attribute value, else null.
	 * @param edge The edge which is searched
	 * @param name The name of the wanted attribute
	 * @return The attribute value or null on found
	 */
	Comparable<?> getPropertyByName(IEdge edge, String name);	
	
	/**
	 * Creates a list of all the properties of the specified edge and returns it.
	 * @param edge The edge which properties are wanted
	 * @return Returns a collection containing all properties of the given edge
	 */
	Map<String, Comparable<?>> getProperties(IEdge edge);
	
	/**
	 * Creates a list of all the property names of the specified edge and returns it.
	 * @param edge The edge which property names are wanted
	 * @return Returns a collection containing all property names of the given edge
	 */
	Collection<Comparable<?>> getPropertyNames(IEdge edge);	
	
	/**
	 * Creates a list of all the property values of the specified edge and returns it.
	 * @param edge The edge which property values are wanted
	 * @return Returns a collection containing all property values of the given edge
	 */
	Collection<Comparable<?>> getPropertyValues(IEdge edge);
	

}
