package org.rentierdb.objectmanagement.graph.edge;

import java.util.Map;

import org.rentierdb.objectmanagement.graph.vertex.IVertex;

/**
 * This interface specifies several CRUD operations executable on ISingleEdges
 * 
 * @author Jan-Christoph Klie
 * 
 */
public interface ISingleEdgeCRUD extends IEdgeCRUD {
	
	//####################Create######################
	
	/**
	 * Creates an edge starting from startVertex to endVertex
	 * @param startVertex The vertex where the created edge starts
	 * @param endVertex The vertex where the created edge ends
	 * @return Returns an edge starting from startVertex to endVertex 
	 */
	IEdge createEdge(IVertex startVertex, IVertex endVertex);
	
	/**
	 * Creates an edge starting from startVertex to endVertex
	 * with the properties specified by the data map 
	 * @param startVertex The vertex where the created edge starts
	 * @param endVertex The vertex where the created edge ends
	 * @param data The properties of the edge
	 * @return Returns an edge starting from startVertex to endVertex with the specified properties
	 */
	IEdge createEdge(IVertex startVertex, IVertex endVertex, Map<String, Comparable<?>> data);
	
	//####################Read######################
	
	IVertex getStart(IEdge edge);
	
	IVertex getEnd(ISingleEdge singleEdge);
	
	//####################Update######################
	
	//####################Delete######################

}
