/**
 * 
 */
package org.rentierdb.simplestorageservice;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.lang3.tuple.Pair;
import org.rentierdb.storageservice.definitions.IGraphElementAddDefinition;
import org.rentierdb.storageservice.definitions.IGraphElementStorageDefinition;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;

/**
 * This is the definition of a to be added graph element.
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation.StorageGraphElement
 */
public class GraphElementAddDefinition implements IGraphElementAddDefinition {

	// ####################Private Member###############

	private String comment = null;

	private HashMap<Long, HashMap<Integer,Pair<? extends Comparable<?>, AttributeType>>> structuredAttributes;

	private HashMap<String, Pair<? extends Comparable<?>, AttributeType>> unstructuredAttributes;

	//#####################Protected Constructor###########
	
	protected GraphElementAddDefinition(){
		structuredAttributes = new HashMap<Long, HashMap<Integer,Pair<? extends Comparable<?>,AttributeType>>>();
		unstructuredAttributes = new HashMap<String, Pair<? extends Comparable<?>,AttributeType>>();
	}
	
	// #####################Public Methods###############

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.definitions.IGraphElementStorageDefinition
	 * #setComment(java.lang.String)
	 */
	@Override
	public void setComment(String myComment) {
		comment = myComment;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(IGraphElementStorageDefinition arg0) {
		// TODO Auto-generated method stub
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.definitions.IGraphElementAddDefinition#
	 * addStructuredAttribute(long, int, java.lang.Comparable)
	 */
	@Override
	public void addStructuredAttribute(long VertexSchemeID, int SchemeMapper, Comparable<?> myAttributeValue, AttributeType myAttributeType) {
		if(structuredAttributes.containsKey(VertexSchemeID))
			structuredAttributes.get(VertexSchemeID).put(SchemeMapper, Pair.of(myAttributeValue,myAttributeType));
		else{
			HashMap<Integer,Pair<? extends Comparable<?>, AttributeType>> temp = new HashMap<Integer,Pair<? extends Comparable<?>, AttributeType>>();
			temp.put(SchemeMapper, Pair.of(myAttributeValue,myAttributeType));
			structuredAttributes.put(VertexSchemeID, temp);
		}	
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.definitions.IGraphElementAddDefinition#
	 * addUnstructuredAttribute(java.lang.String, java.lang.Comparable)
	 */
	@Override
	public void addUnstructuredAttribute(String myAttributeName, Comparable<?> myAttributeValue, AttributeType myAttributeType) {
		if(unstructuredAttributes.containsKey(myAttributeName))
			throw new StorageServiceException("The AttributeName " + myAttributeName + " is already in the list of unstructured attributes!");
		unstructuredAttributes.put(myAttributeName, Pair.of(myAttributeValue, myAttributeType));
	}

	// #####################Private Methods##############

	protected String getComment() {
		return comment;
	}

//	@SuppressWarnings("rawtypes")
//	protected Map<Long, Pair<Integer, Pair<IStorageAttribute, AttributeType>>> getStructuredAttribute() {
//		if(structuredAttributes.isEmpty())
//			return null;
//		HashMap<Long, Pair<Integer, Pair<IStorageAttribute, AttributeType>>> result = new HashMap<Long, Pair<Integer, Pair<IStorageAttribute, AttributeType>>>();
//		for (Entry<Integer, Pair<? extends Comparable<?>, AttributeType>> attribute : structuredAttributes.entrySet()) {
//			IStorageAttribute temp = new StorageAttribute(attribute.getValue().getKey());
//			result.put(structuredSchemeMapper.get(attribute.getKey()), Pair.of(attribute.getKey(), Pair.of(temp, attribute.getValue().getRight())));
//		}
//		return result;
//	}

	
	/**
	 * @return the structuredAttributes
	 */
	protected HashMap<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> getStructuredAttributes() {
		return structuredAttributes;
	}

	/**
	 * @return the unstructuredAttributes
	 */
	protected HashMap<String, Pair<? extends Comparable<?>, AttributeType>> getUnstructuredAttributes() {
		return unstructuredAttributes;
	}
	
	
}
