/**
 * 
 */
package org.rentierdb.simplestorageservice;

import java.util.Collection;
import java.util.Map;

import org.rentierdb.storageservice.exception.StorageServiceGraphElementException;
import org.rentierdb.storageservice.revision.IGraphElementRevision;
import org.rentierdb.storageservice.storage.IStorageAttribute;

/**
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation.StorageGraphElement
 */
public class GraphElementRevision implements IGraphElementRevision {

	// ###############Private Member############

	int currentRevision;

	StorageGraphElement currentGraphElement;

	// ###############Protected Constructor#########

	protected GraphElementRevision(StorageGraphElement myGraphElement, int myRevision) {
		currentRevision = myRevision;
		currentGraphElement = myGraphElement;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.revision.IGraphElementRevision#getRevision()
	 */
	@Override
	public int getRevision() {
		return currentRevision;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.revision.IGraphElementRevision#getCreationDate
	 * ()
	 */
	@Override
	public long getCreationDate() {
		return currentGraphElement.getCreationDate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getModificatonDate()
	 */
	@Override
	public long getModificatonDate() {
		return currentGraphElement.getModificatonDate();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.revision.IGraphElementRevision#getComment()
	 */
	@Override
	public String getComment() {
		return currentGraphElement.getComment();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.revision.IGraphElementRevision#isCurrentRevision
	 * ()
	 */
	@Override
	public boolean isCurrentRevision() {
		if (currentGraphElement.getRevision() != currentRevision)
			return false;
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getCountOfStructuredAttributes()
	 */
	@Override
	public int getCountOfStructuredAttributes() {
		return currentGraphElement.getCountOfStructuredAttributes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getCountOfVertexSchemes()
	 */
	@Override
	public int getCountOfVertexSchemes() {
		return currentGraphElement.getCountOfVertexSchemes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.rentierdb.storageservice.revision.IGraphElementRevision#getVertexSchemes
	 * ()
	 */
	@Override
	public Collection<Long> getVertexSchemes() {
		return currentGraphElement.getVertexSchemes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * derivesFromVertexScheme(long)
	 */
	@Override
	public boolean derivesFromVertexScheme(long myGraphElementID) {
		return currentGraphElement.derivesFromVertexScheme(myGraphElementID);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getAllStructuredAttributesByGraphElementID(long)
	 */
	@Override
	public Map<Integer, IStorageAttribute<?>> getAllStructuredAttributesByGraphElementID(long myGraphElementID) throws StorageServiceGraphElementException {
		return currentGraphElement.getAllStructuredAttributesByGraphElementID(myGraphElementID, currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getAllStructuredAttributes()
	 */
	@Override
	public Map<Long, Map<Integer, IStorageAttribute<?>>> getAllStructuredAttributes() {
		return currentGraphElement.getAllStructuredAttributes(currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getStructuredAttribute(long, int)
	 */
	@Override
	public IStorageAttribute<?> getStructuredAttribute(long myGraphElementID, int mySchemeMapper) throws StorageServiceGraphElementException {
		return currentGraphElement.getStructuredAttributeRevision(myGraphElementID, mySchemeMapper, currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getCountOfUnstructuredAttributes()
	 */
	@Override
	public int getCountOfUnstructuredAttributes() {
		return currentGraphElement.getCountOfUnstructuredAttributes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getUnstructuredAttribute(java.lang.String)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttribute(String myAttributeName) throws StorageServiceGraphElementException {
		return currentGraphElement.getUnstructuredAttribute(myAttributeName, currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getUnstructuredAttribute(int)
	 */
	@Override
	public IStorageAttribute<?> getUnstructuredAttribute(int myAttributeMapper) throws StorageServiceGraphElementException {
		return currentGraphElement.getUnstructuredAttribute(myAttributeMapper, currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getCurrentRevisionNumberOfAttribute(java.lang.String)
	 */
	@Override
	public int getCurrentRevisionNumberOfAttribute(String myAttributeName) {
		return currentGraphElement.getCurrentRevisionNumberOfAttribute(myAttributeName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getCurrentRevisionNumberOfAttribute(int)
	 */
	@Override
	public int getCurrentRevisionNumberOfAttribute(int myAttributeMapper) {
		return currentGraphElement.getCurrentRevisionNumberOfAttribute(myAttributeMapper);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getAllNamesAndMapperOfUnstructuredAttributes()
	 */
	@Override
	public Map<Integer, String> getAllNamesAndMapperOfUnstructuredAttributes() {
		return currentGraphElement.getAllNamesAndMapperOfUnstructuredAttributes();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getAllUnstructuredAttributes()
	 */
	@Override
	public Map<String, IStorageAttribute<?>> getAllUnstructuredAttributes() {
		return currentGraphElement.getAllUnstructuredAttributes(currentRevision);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getUnstructuredAttributeName(int)
	 */
	@Override
	public String getUnstructuredAttributeName(int myAttributeMapper) throws StorageServiceGraphElementException {
		return currentGraphElement.getUnstructuredAttributeName(myAttributeMapper);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * getUnstructuredAttributeMapper(java.lang.String)
	 */
	@Override
	public int getUnstructuredAttributeMapper(String myAttributeName) throws StorageServiceGraphElementException {
		return currentGraphElement.getUnstructuredAttributeMapper(myAttributeName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * hasUnstructuredAttribute(java.lang.String)
	 */
	@Override
	public boolean hasUnstructuredAttribute(String myAttributeName) {
		return currentGraphElement.hasUnstructuredAttribute(myAttributeName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.rentierdb.storageservice.revision.IGraphElementRevision#
	 * hasUnstructuredAttribute(int)
	 */
	@Override
	public boolean hasUnstructuredAttribute(int AttributeMapper) {
		return currentGraphElement.hasUnstructuredAttribute(AttributeMapper);
	}

}
