/**
 * 
 */
package org.rentierdb.simplestorageservice;

import java.util.ArrayList;
import java.util.concurrent.ConcurrentHashMap;

/**
 * This is the vertex store, which holds each graph element.
 * 
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation.SimpleStorageService
 * 
 */
public class GraphElementStore {

	// #############Private Member#########

	private ConcurrentHashMap<Long, StorageGraphElement> graphElementStore;

	private GraphElementIDManager graphIdManager;

	// #############Public Constructor#######

	public GraphElementStore() {
		graphElementStore = new ConcurrentHashMap<Long, StorageGraphElement>();
		graphIdManager = new GraphElementIDManager();
	}

	// #############Public Methods#########

	public long add(StorageGraphElement myGraphElement) {
		long graphElementID = graphIdManager.getID();
		graphElementStore.put(graphElementID, myGraphElement);
		return graphElementID;
	}

	public StorageGraphElement getGraphElement(long myGraphElement) {
		return graphElementStore.get(myGraphElement);
	}

	public int getCountOfGraphElements() {
		return graphElementStore.size();
	}

	public void removeGraphElement(long myGraphElement) {
		graphIdManager.removeID(myGraphElement);
		graphElementStore.remove(myGraphElement);
	}

	public boolean getGraphElementExists(long myGraphElementID) {
		return graphElementStore.contains(myGraphElementID);
	}

	private class GraphElementIDManager {

		// ################Private Member###########

		long currentID;

		ArrayList<Long> assignedIDs;

		// ################Protected Constructor#######

		protected GraphElementIDManager() {
			assignedIDs = new ArrayList<Long>();
			currentID = Long.MIN_VALUE;
		}

		protected GraphElementIDManager(ArrayList<Long> myAssignedIDs) {
			assignedIDs = myAssignedIDs;
			currentID = Long.MIN_VALUE;
		}

		// ###############Protected Methods#########

		protected long getID() {
			while (assignedIDs.contains(currentID)) {
				currentID++;
			}
			assignedIDs.add(currentID);
			return currentID;
		}

		protected void removeID(long myID) {
			if (assignedIDs.contains(myID))
				assignedIDs.remove(myID);
		}

	}

}
