/**
 * 
 */
package org.rentierdb.simplestorageservice;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.lang3.tuple.Pair;
import org.rentierdb.storageservice.definitions.IGraphElementStorageDefinition;
import org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.storage.AttributeType;

/**
 * This is the implementation of an update definition.
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation.SimpleStorageService
 */
public class GraphElementUpdateDefinition extends GraphElementAddDefinition implements
		IGraphElementUpdateDefinition {
	
	// ####################Private Member###############

	private ArrayList<String> toBeDeletedUnstructuredAttributes;
	
	private HashMap<String,Comparable<?>> toBeUpdatedUnstructuredAttributes;
	
	private HashMap<Long, HashMap<Integer,Comparable<?>>> toBeUpdatedStructuredAttributes;
	
	
	//#####################Protected Constructor###########
	
	protected GraphElementUpdateDefinition(){
		super();
		toBeDeletedUnstructuredAttributes = new ArrayList<String>();
		toBeUpdatedUnstructuredAttributes = new HashMap<String, Comparable<?>>();
		toBeUpdatedStructuredAttributes = new HashMap<Long, HashMap<Integer,Comparable<?>>>();
	}
	
	// #####################Public Methods###############

	/* (non-Javadoc)
	 * @see org.rentierdb.storageservice.definitions.IGraphElementStorageDefinition#setComment(java.lang.String)
	 */
	@Override
	public void setComment(String myComment) {
		super.setComment(myComment);
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(IGraphElementStorageDefinition o) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition#updateStructuredAttribute(long, int, java.lang.Comparable)
	 */
	@Override
	public void updateStructuredAttribute(long VertexSchemeID,
			int SchemeMapper, Comparable<?> myAttributeValue) {
		if(toBeUpdatedStructuredAttributes.containsKey(VertexSchemeID)){
			HashMap<Integer, Comparable<?>> attr = toBeUpdatedStructuredAttributes.get(VertexSchemeID);
			if(attr.containsKey(SchemeMapper))
				throw new StorageServiceException("This SchemeMapper has been already updated.");
			attr.put(SchemeMapper, myAttributeValue);
		}
		else{
			HashMap<Integer, Comparable<?>> attr = new HashMap<Integer, Comparable<?>>();
			attr.put(SchemeMapper, myAttributeValue);
			toBeUpdatedStructuredAttributes.put(VertexSchemeID, attr);
		}	

	}

	/* (non-Javadoc)
	 * @see org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition#updateUnstructuredAttribute(java.lang.String, java.lang.Comparable)
	 */
	@Override
	public void updateUnstructuredAttribute(String myAttributeName,
			Comparable<?> myAttributeValue) {
		if(toBeUpdatedUnstructuredAttributes.containsKey(myAttributeName))
			throw new StorageServiceException("This UnstructuredAttribute has been already updated.");
		toBeUpdatedUnstructuredAttributes.put(myAttributeName, myAttributeValue);
	}

	/* (non-Javadoc)
	 * @see org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition#removeUnstructuredAttribute(java.lang.String)
	 */
	@Override
	public void removeUnstructuredAttribute(String myAttributeName) {
		if(!toBeDeletedUnstructuredAttributes.contains(myAttributeName))
			toBeDeletedUnstructuredAttributes.add(myAttributeName);
	}
	
	/**
	 * @return the structuredAttributes
	 */
	protected HashMap<Long, HashMap<Integer, Pair<? extends Comparable<?>, AttributeType>>> getStructuredAttributes() {
		return super.getStructuredAttributes();
	}
	
	/**
	 * @return the unstructuredAttributes
	 */
	protected HashMap<String, Pair<? extends Comparable<?>, AttributeType>> getUnstructuredAttributes() {
		return super.getUnstructuredAttributes();
	}

	/**
	 * @return the toBeDeletedUnstructuredAttributes
	 */
	protected ArrayList<String> getToBeDeletedUnstructuredAttributes() {
		return toBeDeletedUnstructuredAttributes;
	}

	/**
	 * @return the toBeUpdatedUnstructuredAttributes
	 */
	protected HashMap<String, Comparable<?>> getToBeUpdatedUnstructuredAttributes() {
		return toBeUpdatedUnstructuredAttributes;
	}

	/**
	 * @return the toBeUpdatedStructuredAttributes
	 */
	protected HashMap<Long, HashMap<Integer, Comparable<?>>> getToBeUpdatedStructuredAttributes() {
		return toBeUpdatedStructuredAttributes;
	}
	
	
	

}
