/**
 * 
 */
package org.rentierdb.simplestorageservice;

import java.util.ArrayList;
import java.util.List;

import net.xeoh.plugins.base.annotations.PluginImplementation;

import org.rentierdb.storageservice.IStorageService;
import org.rentierdb.storageservice.definitions.IGraphElementAddDefinition;
import org.rentierdb.storageservice.definitions.IGraphElementUpdateDefinition;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.revision.IGraphElementRevision;
import org.rentierdb.storageservice.transaction.ITransactionToken;


/**
 * This is the SimpleStorageService implementation.
 * 
 * @author Michael Schilonka
 * @version 0.1
 * @category Implementation. StorageService
 * 
 */
@PluginImplementation
public final class SimpleStorageService implements IStorageService {

	// #################Private Member#############

	private GraphElementStore graphElementStore;

	// ################# Constructor##############

	public SimpleStorageService() {
		graphElementStore = new GraphElementStore();
	}

	// #################Public Methods#############

	@Override
	public long getCountOfVertices() {
		return (int) graphElementStore.getCountOfGraphElements();
	}

	@Override
	public boolean getGraphElementExists(ITransactionToken myTransactionToken, long myGraphElementID) {
		return graphElementStore.getGraphElementExists(myGraphElementID);
	}

	@Override
	public IGraphElementRevision getGraphElementCurrentRevision(ITransactionToken myTransactionToken, long myGraphElementID) throws StorageServiceException {
		StorageGraphElement handle = graphElementStore.getGraphElement(myGraphElementID);
		return new GraphElementRevision(handle, handle.getRevision());
	}

	@Override
	public IGraphElementRevision getGraphElementOfRevision(ITransactionToken myTransactionToken, long myGraphElementID, int myRevisionID)
			throws StorageServiceException {
		StorageGraphElement handle = graphElementStore.getGraphElement(myGraphElementID);
		return new GraphElementRevision(handle, myRevisionID);
	}

	@Override
	public List<IGraphElementRevision> getGraphElementsCurrentRevision(ITransactionToken myTransactionToken, List<Long> myInterestingGraphElementIDs)
			throws StorageServiceException {
		ArrayList<IGraphElementRevision> result = new ArrayList<IGraphElementRevision>();
		for (Long graphElement : myInterestingGraphElementIDs) {
			StorageGraphElement handle = graphElementStore.getGraphElement(graphElement);
			result.add(new GraphElementRevision(handle, handle.getRevision()));
		}
		return result;
	}

	@Override
	public List<IGraphElementRevision> getGraphElementsOfRevision(ITransactionToken myTransactionToken, List<Long> myInterestingGraphElementIDs, int myRevision)
			throws StorageServiceException {
		ArrayList<IGraphElementRevision> result = new ArrayList<IGraphElementRevision>();
		
		return result;
	}

	@Override
	public void removeGraphElementCurrentRevision(ITransactionToken myTransactionToken, long myGraphElementID) throws StorageServiceException {
		removeGraphElementOfRevision(myTransactionToken, myGraphElementID, graphElementStore.getGraphElement(myGraphElementID).getRevision());

	}

	@Override
	public void removeGraphElementOfRevision(ITransactionToken myTransactionToken, long myGraphElementID, int myRevisionID) throws StorageServiceException {
		if (myRevisionID == 1) {
			graphElementStore.removeGraphElement(myGraphElementID);
		}
		// TODO add code for revision downgrade

	}

	@Override
	public long addGraphElement(ITransactionToken myTransactionToken, IGraphElementAddDefinition myAddDefinition) throws StorageServiceException {
		return graphElementStore.add(new StorageGraphElement((GraphElementAddDefinition) myAddDefinition));
	}

	@Override
	public void updateGraphElement(ITransactionToken myTransactionToken, long myGraphElementID, IGraphElementUpdateDefinition myUpdateDefinition)
			throws StorageServiceException {
		graphElementStore.getGraphElement(myGraphElementID).updateGraphElement(myUpdateDefinition);
		
	}

	@Override
	public ITransactionToken beginTransaction() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void endTransaction(ITransactionToken myTransaction) {
		// TODO Auto-generated method stub

	}

	@Override
	public void rollbackTransaction(ITransactionToken myTransaction) {
		// TODO Auto-generated method stub

	}

	@Override
	public void shutdown() {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean supportsTransaction() {
		return false;
	}

	@Override
	public long getNumberOfBytes() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public IGraphElementAddDefinition createAddDefinition() {
		return new GraphElementAddDefinition();
	}

	@Override
	public IGraphElementUpdateDefinition createUpdateDefinition() {
		return new GraphElementUpdateDefinition();
	}

	

}
