/**
 * 
 */
package org.rentierdb.simplestorageservice;

import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;


/**
 * This is a generic StorageAttribute, which holds the atomic value of an
 * attribute.
 * 
 * @author Michael Schilonka
 * @version 0.1
 * @param <T>
 *            The type of the attribute value.
 * @category Implementation.SimpleStorageService.GraphElements
 */
public class StorageAttribute<T extends Comparable<T>> implements
		IStorageAttribute<T> {

	// #######Private Member########
	private static final long serialVersionUID = 5814474201991296766L;

	private T attributeValue;

	private AttributeType attributeType;
	

	// #######Protected Methods######

	protected StorageAttribute(T myObjectValue) {
		this.attributeValue = myObjectValue;
		this.attributeType = AttributeType.SIMPLE;
	}

	protected StorageAttribute(T myObjectValue, AttributeType myAttrbuteType) {
		if(myAttrbuteType.equals(AttributeType.EDGE)){
			if(!(myObjectValue instanceof Long))
				throw new StorageServiceException("StorageAttributes with the type EDGE have to be long value type!");
		}
		
		this.attributeValue = myObjectValue;
		this.attributeType = myAttrbuteType;
			
	}

	// #######Public Methods########

	@Override
	public int compareTo(T myObject) {
		return this.attributeValue.compareTo(myObject);
	}
	
	public int compareTo(IStorageAttribute<T> myObject){
		return compareTo((T) myObject.getRawObject());				
	}

	@Override
	public AttributeType getAttributeType() {
		return this.attributeType;
	}

	@Override
	public T getRawObject() {
		return this.attributeValue;
	}

	public boolean equals(Object myObject) {
		return this.attributeValue.equals(((IStorageAttribute<T>) myObject)
				.getRawObject());
	}

	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}
}
