package org.rentierdb.simplestorageservice;

import java.math.BigInteger;

import junit.framework.TestCase;

import org.junit.Before;
import org.junit.Test;
import org.rentierdb.storageservice.exception.StorageServiceException;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;

public class StorageAttributeTest extends TestCase {

	IStorageAttribute<String> attr11;
	IStorageAttribute<String> attr12;
	IStorageAttribute<Integer> attr21;
	IStorageAttribute<Integer> attr22;
	IStorageAttribute<Long> attr31;
	IStorageAttribute<Long> attr32;
	IStorageAttribute<BigInteger> attr41;
	IStorageAttribute<BigInteger> attr42;
	String testString = "TEST_String_23456_#+987766_qwertzui";
	int testInt = 42;
	long testLong = 42L;
	BigInteger testBig;

	@Before
	public void setUp() throws Exception {
		testBig = new BigInteger("12345678998765436214");
		attr11 = new StorageAttribute<String>(testString);
		attr21 = new StorageAttribute<Integer>(testInt);
		attr31 = new StorageAttribute<Long>(testLong);
		attr41 = new StorageAttribute<BigInteger>(testBig);
	}

	@Test(expected=StorageServiceException.class)
	public void testStorageAttribute(){
		StorageAttribute<String>attr111 = new StorageAttribute<String>("Test");
		attr11 = new StorageAttribute<String>("Test", AttributeType.SIMPLE);
		attr111 = new StorageAttribute<String>("String", AttributeType.EDGE);
		StorageAttribute<Long> attr112 = new StorageAttribute<Long>(12124L,AttributeType.EDGE);
	}
	
	
	@Test(expected=NullPointerException.class)
	public void testCompareTo() {
		assertEquals(0, attr11.compareTo(testString));
		assertEquals(0, attr11.compareTo(attr11));
		attr12 = new StorageAttribute<String>("OtherString");
		assertEquals(5, attr11.compareTo(attr12));
		String test = null;
		assertEquals(0, attr11.compareTo(test));
		
		attr22 = null;
		assertEquals(0, attr21.compareTo(attr22));	
	}

	@Test
	public void testGetAttributeType() {
		IStorageAttribute<Integer> attr21 = new StorageAttribute<Integer>(
				123215);
		IStorageAttribute<Long> attr32 = new StorageAttribute<Long>(testLong,
				AttributeType.ADVANCED);
		IStorageAttribute<BigInteger> attr42 = new StorageAttribute<BigInteger>(
				new BigInteger("5465789798"), AttributeType.SIMPLE);
		assertEquals(AttributeType.SIMPLE,attr11.getAttributeType());
		//assertEquals(AttributeType.EDGE,attr21.getAttributeType());
		assertEquals( AttributeType.SIMPLE, attr31.getAttributeType());
		assertEquals( AttributeType.SIMPLE, attr41.getAttributeType());
		assertEquals( AttributeType.ADVANCED,attr32.getAttributeType());
		assertEquals(AttributeType.SIMPLE, attr42.getAttributeType());
	}

	@Test
	public void testGetRawObject() {
		assertEquals(42, attr21.getRawObject().intValue());
		assertEquals(testLong, attr31.getRawObject().longValue());
	}

	@Test
	public void testEqualsObject() {
		IStorageAttribute<Integer> attr21 = new StorageAttribute<Integer>(123215);
		IStorageAttribute<Long> attr32 = new StorageAttribute<Long>(testLong, AttributeType.ADVANCED);
		IStorageAttribute<Long> attr31 = new StorageAttribute<Long>(testLong, AttributeType.ADVANCED);
		
		assertEquals(false, attr21.equals(attr22));
		assertEquals(false, attr11.equals(attr22));
		assertEquals(false, attr11.equals(attr12));

		assertEquals(true, attr32.equals(attr31));
		
		assertEquals(false, attr31.equals(null));
		assertEquals(false, attr32.equals(testBig));
		
		assertEquals(false, attr21.equals(attr32));
		
		assertEquals(false, attr41.equals(new StorageAttribute<BigInteger>(
				new BigInteger("564654654"), AttributeType.SIMPLE)));
		assertEquals(false, attr41.equals(new StorageAttribute<BigInteger>(
				new BigInteger("5465789798"), AttributeType.ADVANCED)));
		//assertEquals(false, attr41.equals(new StorageAttribute<BigInteger>(
		//		new BigInteger("5465789798"), AttributeType.EDGE)));
		assertEquals(true, attr41.equals(new StorageAttribute<BigInteger>(
				testBig)));
		assertEquals(true, attr41.equals(testBig));
		
		assertEquals(true, attr31.equals(attr31));
	}

	@Test
	public void testClone() throws CloneNotSupportedException {
		IStorageAttribute<Long> attr21 = new StorageAttribute<Long>(123215L, AttributeType.EDGE);
		IStorageAttribute<Integer> attr23 = (IStorageAttribute<Integer>)attr21.clone();
		assertEquals(attr23.equals(attr21), true);
	}

}
