package org.rentierdb.simplestorageservice;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import junit.framework.TestCase;

import org.junit.Before;
import org.junit.Test;
import org.rentierdb.storageservice.storage.AttributeType;
import org.rentierdb.storageservice.storage.IStorageAttribute;

public class StorageGraphElementTest extends TestCase {

	StorageGraphElement aStorageGraphElement;

	@Before
	public void setUp() throws Exception {
		aStorageGraphElement = new StorageGraphElement(generateAddDefinition());
	}

	@Test
	public void testStorageGraphElement() {
		assertTrue(aStorageGraphElement != null);
	}

	//
	// @Test
	// public void testUpdateGraphElement() {
	// fail("Not yet implemented");
	// }

	// @Test
	// public void testClone() {
	// fail("Not yet implemented");
	// }

	@Test
	public void testGetRevision() {

		int revision = aStorageGraphElement.getRevision();
		assertEquals(1, revision);
	}

	@Test
	public void testGetCreationDate() {
		StorageGraphElement aStorageGraphElement = new StorageGraphElement(generateAddDefinition());
		assertTrue(System.currentTimeMillis() == aStorageGraphElement.getCreationDate());
	}

	// @Test
	// public void testGetModificatonDate() {
	// fail("Not yet implemented");
	// }

	@Test
	public void testGetComment() {
		assertTrue("This is a simple test GraphElementAddDefintion".equals(aStorageGraphElement.getComment()));
	}

	@Test
	public void testGetCountOfStructuredAttributes() {
		assertEquals(4, aStorageGraphElement.getCountOfStructuredAttributes());
	}

	@Test
	public void testGetCountOfVertexSchemes() {
		assertEquals(2, aStorageGraphElement.getCountOfVertexSchemes());
	}

	@Test
	public void testGetVertexSchemes() {
		Collection<Long> expected = new ArrayList<Long>();
		expected.add(4711L);
		expected.add(4712L);
		Collection<Long> actual = aStorageGraphElement.getVertexSchemes();

		assertEquals(expected.size(), actual.size());
		for (Long long1 : expected) {
			if (!actual.contains(long1))
				fail("VertexScheme missmatch!");
		}

	}

	@Test
	public void testDerivesFromVertexScheme() {
		assertTrue(aStorageGraphElement.derivesFromVertexScheme(4711L));
		assertTrue(aStorageGraphElement.derivesFromVertexScheme(4712L));
	}

	@Test
	public void testGetAllStructuredAttributesByGraphElementIDCurrentRevision() {
		HashMap<Integer, IStorageAttribute<?>> expected = new HashMap<Integer, IStorageAttribute<?>>();
		expected.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		expected.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		expected.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));

		Map<Integer, IStorageAttribute<?>> actual = aStorageGraphElement.getAllStructuredAttributesByGraphElementIDCurrentRevision(4711L);
		assertEquals(expected.size(), actual.size());
		for (Entry<Integer, IStorageAttribute<?>> value : expected.entrySet()) {
			if (!actual.containsKey(value.getKey()))
				fail("GraphElementID is not in list!");
			assertEquals(value.getValue(), actual.get(value.getKey()));
		}

		actual = aStorageGraphElement.getAllStructuredAttributesByGraphElementIDCurrentRevision(4712L);
		expected = new HashMap<Integer, IStorageAttribute<?>>();
		expected.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		assertEquals(expected.size(), actual.size());
		for (Entry<Integer, IStorageAttribute<?>> value : expected.entrySet()) {
			if (!actual.containsKey(value.getKey()))
				fail("GraphElementID is not in list!");
			if (!value.getValue().equals(actual.get(value.getKey())))
				fail("Mismatch!");
		}
	}

	@Test
	public void testGetAllStructuredAttributesByGraphElementID() {

		HashMap<Integer, IStorageAttribute<?>> expected = new HashMap<Integer, IStorageAttribute<?>>();
		expected.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		expected.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		expected.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));

		Map<Integer, IStorageAttribute<?>> actual = aStorageGraphElement.getAllStructuredAttributesByGraphElementID(4711L, 1);
		assertEquals(expected.size(), actual.size());
		for (Entry<Integer, IStorageAttribute<?>> value : expected.entrySet()) {
			if (!actual.containsKey(value.getKey()))
				fail("GraphElementID is not in list!");
			assertEquals(value.getValue(), actual.get(value.getKey()));
		}

		actual = aStorageGraphElement.getAllStructuredAttributesByGraphElementID(4712L, 1);
		expected = new HashMap<Integer, IStorageAttribute<?>>();
		expected.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllStructuredAttributesCurrentRevision() {
		Map<Long, Map<Integer, IStorageAttribute<?>>> expected = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
		HashMap<Integer, IStorageAttribute<?>> value1 = new HashMap<Integer, IStorageAttribute<?>>();
		value1.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		value1.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		value1.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));
		HashMap<Integer, IStorageAttribute<?>> value2 = new HashMap<Integer, IStorageAttribute<?>>();
		value2.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		expected.put(4711L, value1);
		expected.put(4712L, value2);
		Map<Long, Map<Integer, IStorageAttribute<?>>> actual = aStorageGraphElement.getAllStructuredAttributesCurrentRevision();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllStructuredAttributes() {
		Map<Long, Map<Integer, IStorageAttribute<?>>> expected = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
		HashMap<Integer, IStorageAttribute<?>> value1 = new HashMap<Integer, IStorageAttribute<?>>();
		value1.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		value1.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		value1.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));
		HashMap<Integer, IStorageAttribute<?>> value2 = new HashMap<Integer, IStorageAttribute<?>>();
		value2.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		expected.put(4711L, value1);
		expected.put(4712L, value2);
		Map<Long, Map<Integer, IStorageAttribute<?>>> actual = aStorageGraphElement.getAllStructuredAttributes(1);

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllStructuredAttributesFirstRevision() {
		Map<Long, Map<Integer, IStorageAttribute<?>>> expected = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
		HashMap<Integer, IStorageAttribute<?>> value1 = new HashMap<Integer, IStorageAttribute<?>>();
		value1.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		value1.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		value1.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));
		HashMap<Integer, IStorageAttribute<?>> value2 = new HashMap<Integer, IStorageAttribute<?>>();
		value2.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		expected.put(4711L, value1);
		expected.put(4712L, value2);
		Map<Long, Map<Integer, IStorageAttribute<?>>> actual = aStorageGraphElement.getAllStructuredAttributesFirstRevision();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetStructuredAttributeCurrentRevision() {
		Map<Long, Map<Integer, IStorageAttribute<?>>> expected = new HashMap<Long, Map<Integer, IStorageAttribute<?>>>();
		HashMap<Integer, IStorageAttribute<?>> value1 = new HashMap<Integer, IStorageAttribute<?>>();
		value1.put(1, new StorageAttribute("TestString", AttributeType.SIMPLE));
		value1.put(2, new StorageAttribute("TestString1", AttributeType.SIMPLE));
		value1.put(3, new StorageAttribute(URI.create("http://www.schilonka.de"), AttributeType.SIMPLE));
		HashMap<Integer, IStorageAttribute<?>> value2 = new HashMap<Integer, IStorageAttribute<?>>();
		value2.put(1, new StorageAttribute(4723L, AttributeType.EDGE));
		expected.put(4711L, value1);
		expected.put(4712L, value2);
		Map<Long, Map<Integer, IStorageAttribute<?>>> actual = aStorageGraphElement.getAllStructuredAttributesFirstRevision();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetStructuredAttributeRevision() {
		StorageAttribute expected = new StorageAttribute("TestString", AttributeType.SIMPLE);
		IStorageAttribute<?> actual = aStorageGraphElement.getStructuredAttributeRevision(4711L, 1, 1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(4723L, AttributeType.EDGE);
		actual = aStorageGraphElement.getStructuredAttributeRevision(4712L, 1, 1);
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetCountOfUnstructuredAttributes() {
		int actual = aStorageGraphElement.getCountOfUnstructuredAttributes();
		assertEquals(3, actual);
	}

	@Test
	public void testGetUnstructuredAttributeCurrentRevisionString() {
		StorageAttribute expected = new StorageAttribute("Jochen", AttributeType.SIMPLE);
		IStorageAttribute<?> actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision("Name");
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(12, AttributeType.SIMPLE);
		actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision("Age");
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(7413L, AttributeType.EDGE);
		actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision("isFriendOf");
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetUnstructuredAttributeCurrentRevisionInt() {
		StorageAttribute expected = new StorageAttribute("Jochen", AttributeType.SIMPLE);
		IStorageAttribute<?> actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision(0);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(12, AttributeType.SIMPLE);
		actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision(1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(7413L, AttributeType.EDGE);
		actual = aStorageGraphElement.getUnstructuredAttributeCurrentRevision(2);
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetUnstructuredAttributeIntInt() {
		StorageAttribute expected = new StorageAttribute("Jochen", AttributeType.SIMPLE);
		IStorageAttribute<?> actual = aStorageGraphElement.getUnstructuredAttribute(0, 1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(12, AttributeType.SIMPLE);
		actual = aStorageGraphElement.getUnstructuredAttribute(1, 1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(7413L, AttributeType.EDGE);
		actual = aStorageGraphElement.getUnstructuredAttribute(2, 1);
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetUnstructuredAttributeStringInt() {
		StorageAttribute expected = new StorageAttribute("Jochen", AttributeType.SIMPLE);
		IStorageAttribute<?> actual = aStorageGraphElement.getUnstructuredAttribute("Name", 1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(12, AttributeType.SIMPLE);
		actual = aStorageGraphElement.getUnstructuredAttribute("Age", 1);
		assertTrue(actual.equals(expected));

		expected = new StorageAttribute(7413L, AttributeType.EDGE);
		actual = aStorageGraphElement.getUnstructuredAttribute("isFriendOf", 1);
		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetCurrentRevisionNumberOfAttributeString() {
		int actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute("Name");
		assertEquals(1, actual);

		actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute("Age");
		assertEquals(1, actual);

		actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute("isFriendOf");
		assertEquals(1, actual);
	}

	@Test
	public void testGetCurrentRevisionNumberOfAttributeInt() {
		int actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute(0);
		assertEquals(1, actual);

		actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute(1);
		assertEquals(1, actual);

		actual = aStorageGraphElement.getCurrentRevisionNumberOfAttribute(2);
		assertEquals(1, actual);
	}

	@Test
	public void testGetAllNamesAndMapperOfUnstructuredAttributes() {
		HashMap<Integer, String> expected = new HashMap<Integer, String>();
		expected.put(0, "Name");
		expected.put(1, "Age");
		expected.put(2, "isFriendOf");
		Map<Integer, String> actual = aStorageGraphElement.getAllNamesAndMapperOfUnstructuredAttributes();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllUnstructuredAttributesCurrentRevision() {
		HashMap<String, IStorageAttribute<?>> expected = new HashMap<String, IStorageAttribute<?>>();
		expected.put("Name", new StorageAttribute("Jochen", AttributeType.SIMPLE));
		expected.put("Age", new StorageAttribute(12, AttributeType.SIMPLE));
		expected.put("isFriendOf", new StorageAttribute(7413L, AttributeType.EDGE));
		Map<String, IStorageAttribute<?>> actual = aStorageGraphElement.getAllUnstructuredAttributesCurrentRevision();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllUnstructuredAttributes() {
		HashMap<String, IStorageAttribute<?>> expected = new HashMap<String, IStorageAttribute<?>>();
		expected.put("Name", new StorageAttribute("Jochen", AttributeType.SIMPLE));
		expected.put("Age", new StorageAttribute(12, AttributeType.SIMPLE));
		expected.put("isFriendOf", new StorageAttribute(7413L, AttributeType.EDGE));
		Map<String, IStorageAttribute<?>> actual = aStorageGraphElement.getAllUnstructuredAttributes(1);

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetAllUnstructuredAttributesFirstRevision() {
		HashMap<String, IStorageAttribute<?>> expected = new HashMap<String, IStorageAttribute<?>>();
		expected.put("Name", new StorageAttribute("Jochen", AttributeType.SIMPLE));
		expected.put("Age", new StorageAttribute(12, AttributeType.SIMPLE));
		expected.put("isFriendOf", new StorageAttribute(7413L, AttributeType.EDGE));
		Map<String, IStorageAttribute<?>> actual = aStorageGraphElement.getAllUnstructuredAttributesFirstRevision();

		assertTrue(actual.equals(expected));
	}

	@Test
	public void testGetUnstructuredAttributeName() {
		assertEquals("Name", aStorageGraphElement.getUnstructuredAttributeName(0));
		assertEquals("Age", aStorageGraphElement.getUnstructuredAttributeName(1));
		assertEquals("isFriendOf", aStorageGraphElement.getUnstructuredAttributeName(2));
	}

	@Test
	public void testGetUnstructuredAttributeMapper() {
		assertEquals(0, aStorageGraphElement.getUnstructuredAttributeMapper("Name"));
		assertEquals(1, aStorageGraphElement.getUnstructuredAttributeMapper("Age"));
		assertEquals(2, aStorageGraphElement.getUnstructuredAttributeMapper("isFriendOf"));
	}

	@Test
	public void testHasUnstructuredAttributeString() {
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("Name"));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("Age"));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("isFriendOf"));
		assertTrue(!aStorageGraphElement.hasUnstructuredAttribute("isEnemyOf"));
		assertTrue(!aStorageGraphElement.hasUnstructuredAttribute("Size"));
	}

	@Test
	public void testHasUnstructuredAttributeInt() {
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute(0));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute(1));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute(2));
		assertTrue(!aStorageGraphElement.hasUnstructuredAttribute(3));
		assertTrue(!aStorageGraphElement.hasUnstructuredAttribute(1000));
	}

	@Test
	public void testUpdateGraphStorageElement() {
		aStorageGraphElement.updateGraphElement(generateUpdateDefinition());
		
		assertEquals("This is a simple test GraphElementAddDefintion, but updated", aStorageGraphElement.getComment());
		assertEquals(2, aStorageGraphElement.getRevision());
		
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("Name"));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("Age"));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("isFriendOf"));
		assertTrue(aStorageGraphElement.hasUnstructuredAttribute("isEnemyOf"));
		assertTrue(!aStorageGraphElement.hasUnstructuredAttribute("Size"));
		
		assertEquals(0, aStorageGraphElement.getUnstructuredAttributeMapper("Name"));
		assertEquals(1, aStorageGraphElement.getUnstructuredAttributeMapper("Age"));
		assertEquals(2, aStorageGraphElement.getUnstructuredAttributeMapper("isFriendOf"));
		assertEquals(3, aStorageGraphElement.getUnstructuredAttributeMapper("isEnemyOf"));
		assertEquals(4, aStorageGraphElement.getUnstructuredAttributeMapper("Address"));
		assertEquals(5, aStorageGraphElement.getUnstructuredAttributeMapper("ZipCode"));
	
		
		assertEquals("Name", aStorageGraphElement.getUnstructuredAttributeName(0));
		assertEquals("isFriendOf", aStorageGraphElement.getUnstructuredAttributeName(2));
		assertEquals("Address", aStorageGraphElement.getUnstructuredAttributeName(4));
		assertEquals("ZipCode", aStorageGraphElement.getUnstructuredAttributeName(5));
		assertEquals("isEnemyOf", aStorageGraphElement.getUnstructuredAttributeName(3));
		

		
		
		HashMap<String, IStorageAttribute<?>> expected = new HashMap<String, IStorageAttribute<?>>();
		expected.put("Name", new StorageAttribute("Jochen", AttributeType.SIMPLE));
		expected.put("Age", new StorageAttribute(12, AttributeType.SIMPLE));
		expected.put("isFriendOf", new StorageAttribute(7413L, AttributeType.EDGE));
		Map<String, IStorageAttribute<?>> actual = aStorageGraphElement.getAllUnstructuredAttributesFirstRevision();

		assertTrue(actual.equals(expected));
		
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute("Name", 2).getRawObject());
		assertEquals("Jochen", aStorageGraphElement.getUnstructuredAttribute("Name", 1).getRawObject());
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute("Name", 3).getRawObject());
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute("Name", 500).getRawObject());
		
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute(0, 2).getRawObject());
		assertEquals("Jochen", aStorageGraphElement.getUnstructuredAttribute(0, 1).getRawObject());
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute(0, 3).getRawObject());
		assertEquals("Peter", aStorageGraphElement.getUnstructuredAttribute(0, 500).getRawObject());
		
		
		assertEquals(12, aStorageGraphElement.getUnstructuredAttribute("Age", 1).getRawObject());
		assertEquals(AttributeType.REMOVED, aStorageGraphElement.getUnstructuredAttribute("Age", 2).getAttributeType());
		
		assertEquals(7413L, aStorageGraphElement.getUnstructuredAttribute("isFriendOf", 1).getRawObject());
		assertEquals(7414L, aStorageGraphElement.getUnstructuredAttribute("isFriendOf", 2).getRawObject());
		
		assertEquals(68239, aStorageGraphElement.getUnstructuredAttribute("ZipCode", 2).getRawObject());
		try {
			aStorageGraphElement.getUnstructuredAttribute("ZipCode", 1).getRawObject();
		} catch (Exception e) {
			assertTrue(e != null);
		}
		
		assertEquals("TestString2", aStorageGraphElement.getStructuredAttributeRevision(4713L, 1, 2).getRawObject());
		assertEquals("TestString3", aStorageGraphElement.getStructuredAttributeRevision(4713L, 2, 2).getRawObject());
		assertEquals(URI.create("http://www.schilonka.de"), aStorageGraphElement.getStructuredAttributeRevision(4713L, 3, 2).getRawObject());
		try {
			aStorageGraphElement.getStructuredAttributeRevision(4713L, 3, 1).getRawObject();
		} catch (Exception e) {
			assertTrue(e != null);
		}
		
		assertEquals("TestString", aStorageGraphElement.getStructuredAttributeRevision(4711L, 1, 1).getRawObject());
		assertEquals("YetAnotherTestString", aStorageGraphElement.getStructuredAttributeRevision(4711L, 1, 2).getRawObject());
		assertEquals("YetAnotherTestString", aStorageGraphElement.getStructuredAttributeRevision(4711L, 1, 5).getRawObject());
		assertEquals(URI.create("www.holds.de"), aStorageGraphElement.getStructuredAttributeRevision(4711L, 3, 2).getRawObject());
		assertEquals(URI.create("http://www.schilonka.de"), aStorageGraphElement.getStructuredAttributeRevision(4711L, 3, 1).getRawObject());
		assertEquals(URI.create("www.holds.de"), aStorageGraphElement.getStructuredAttributeRevision(4711L, 3, 10000).getRawObject());
		
		assertEquals("YetAnotherTestString", aStorageGraphElement.getStructuredAttributeCurrentRevision(4711L, 1).getRawObject());
		
		Collection<Long> expectedC = new ArrayList<Long>();
		expectedC.add(4711L);
		expectedC.add(4712L);
		expectedC.add(4713L);
		Collection<Long> actualC = aStorageGraphElement.getVertexSchemes();

		assertEquals(expectedC.size(), actual.size());
		for (Long long1 : expectedC) {
			if (!actualC.contains(long1))
				fail("VertexScheme missmatch!");
		}
		
		assertEquals(8, aStorageGraphElement.getCountOfStructuredAttributes());
		assertEquals(8, aStorageGraphElement.getCountOfStructuredAttributes());
		
	}
	
	private static GraphElementAddDefinition generateAddDefinition() {
		GraphElementAddDefinition aGraphElementAddDefinition = new GraphElementAddDefinition();

		aGraphElementAddDefinition.setComment("This is a simple test GraphElementAddDefintion");

		aGraphElementAddDefinition.addStructuredAttribute(4711L, 1, "TestString", AttributeType.SIMPLE);
		aGraphElementAddDefinition.addStructuredAttribute(4711L, 2, "TestString1", AttributeType.SIMPLE);
		aGraphElementAddDefinition.addStructuredAttribute(4711L, 3, URI.create("http://www.schilonka.de"), AttributeType.SIMPLE);
		aGraphElementAddDefinition.addStructuredAttribute(4712L, 1, 4723L, AttributeType.EDGE);

		aGraphElementAddDefinition.addUnstructuredAttribute("Name", "Jochen", AttributeType.SIMPLE);
		aGraphElementAddDefinition.addUnstructuredAttribute("Age", 12, AttributeType.SIMPLE);
		aGraphElementAddDefinition.addUnstructuredAttribute("isFriendOf", 7413L, AttributeType.EDGE);

		return aGraphElementAddDefinition;
	}

	private GraphElementUpdateDefinition generateUpdateDefinition() {
		GraphElementUpdateDefinition aGraphElementUpdateDefinition = new GraphElementUpdateDefinition();

		aGraphElementUpdateDefinition.setComment("This is a simple test GraphElementAddDefintion, but updated");

		aGraphElementUpdateDefinition.addStructuredAttribute(4713L, 1, "TestString2", AttributeType.SIMPLE);
		aGraphElementUpdateDefinition.addStructuredAttribute(4713L, 2, "TestString3", AttributeType.SIMPLE);
		aGraphElementUpdateDefinition.addStructuredAttribute(4713L, 3, URI.create("http://www.schilonka.de"), AttributeType.SIMPLE);
		aGraphElementUpdateDefinition.addStructuredAttribute(4714L, 1, 4724L, AttributeType.EDGE);

		aGraphElementUpdateDefinition.addUnstructuredAttribute("Address", "Main Street", AttributeType.SIMPLE);
		aGraphElementUpdateDefinition.addUnstructuredAttribute("ZipCode", 68239, AttributeType.SIMPLE);
		aGraphElementUpdateDefinition.addUnstructuredAttribute("isEnemyOf", 7414L, AttributeType.EDGE);

		aGraphElementUpdateDefinition.updateUnstructuredAttribute("Name", "Peter");
		aGraphElementUpdateDefinition.updateUnstructuredAttribute("isFriendOf", 7414L);

		aGraphElementUpdateDefinition.updateStructuredAttribute(4711L, 1, "YetAnotherTestString");
		aGraphElementUpdateDefinition.updateStructuredAttribute(4711L, 2, "YetAnotherTestString1");
		aGraphElementUpdateDefinition.updateStructuredAttribute(4711L, 3, URI.create("www.holds.de"));

		aGraphElementUpdateDefinition.removeUnstructuredAttribute("Age");
	
		return aGraphElementUpdateDefinition;
	}

}
